subroutine COOmatvec(vals, rows, cols, nrow, ncol, nnz, x, y)
!--------------------------------------------------------------------------------
! Compute a sparse matrix times a vector y = A*x, using the dot product form of
! multiplication. A is stored in coordinatewise format (COO).
!
! On entry:
!   nrow  = number of rows in A
!   ncol  = number of columns in A. Only used here to allow compiler
!           length checks in the declaraation of x.
!   x = real array of length equal to the number of columns
!       in the A matrix (AKA ncol)
!   vals, cols, rows = input matrix in coordinatewise format
!
! On return:
!   y = array of length nrow, containing the product y = Ax
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!--------------------------------------------------------------------------------

    ! Define integer and real kinds 
    use kindlytypes, only: int4 => integer4, real8
    use iso_fortran_env, only: stdout => output_unit, &
                               stderr => error_unit
    use noisy
    implicit none
    
    integer(kind=int4), intent(in) :: nrow, ncol, nnz
    integer(kind=int4), intent(in) :: cols(nnz), rows(nnz)
    real(kind=real8), intent(in) :: x(ncol), vals(nnz) 
    real(kind=real8), intent(out):: y(nrow)

    ! local variable
    integer(kind = int4) :: i

    y(1:nrow) = 0.0_real8

    if (benoisy > 1) then
        write(stdout,fmt=200) 'bmux now has nrow = ', nrow
        write(stdout,fmt=200) 'bmux now has ncol = ', ncol
        write(stdout,fmt=200) 'bmux now has nnz = ', nnz
    end if

    if (benoisy > 2) then
        write(stdout,fmt=500) 'bmux: rows = ', rows
        write(stdout,fmt=500) 'bmux: cols = ', cols
        write(stdout,fmt=600) 'bmux: vals = ', vals
    end if

    do i = 1, nnz
        y(rows(i)) = y(rows(i)) + vals(i)*x(cols(i))
    end do

    return
    200 format(a, i0)
    500 format(a, *(/,3x, i0))  ! infinite repeat descriptor
    600 format(a, *(/,3x, es27.17))


end subroutine COOmatvec
