subroutine getsten(x,y,meshpoints,stencil)
!------------------------------------------------------------------------------------
! Return the 5-point centered difference stencil for a 2D second order linear PDE.
! More precisely, for the linear PDE
!     au_xx + bu_yy + cu_z + du_y + eu = f
! on the unit square, where all of a - f are functions of (x,y), give the values for
! the stencil oriented in the standard position:
!
!
!	                    st(5)
!	                     |
!	                     |  
!	                     |
!	                     |
!	                     |
!	                     | 
!	  st(2) ----------- st(1) ---------- st(4)
!	                     |
!	                     |
!	                     |
!	                     |
!	                     |
!	                     |
!	                    st(3)
!
!
! stencil(6) is the right hand side vector component from source terms. If nonzero
! Dirichlet boundary conditions are present, they will need to be added to the
! stencil(6) component appropriately by whoever calls this subroutine.
!
! This is set on domain [0,meshpoints+1] x [0,meshpoints+1] with mesh spacing h =
! 1/(meshpoints+1), and the functions afun-ffun are presumed to be on [0,1]x[0,1].
! The change of variables is reflected in the a-f computations. This is done so that
! the unknowns are on an meshpoints x meshpoints grid.
!
! Unlike the other stuff in this pile of codes, this does not use the noisy module
! and does no I/O. Not much to screw up here, so I've left that off.
!-----------------
! Randall Bramley
! Department of Computer Science
! Indiana University, Bloomington
!------------------------------------------------------------------------------------
    use kindlytypes, only: real8
    use iso_fortran_env, only: stdout => output_unit, &
                               stderr => error_unit
    use coeffs
    use directions
    real(kind=real8) :: h, stencil(6), x, y
    real(kind=real8) :: a, b, c, d, e, f
    real(kind=real8), parameter :: one = 1.0_real8, half = 0.5_real8

    a = afun(x,y)
    b = bfun(x,y)
    c = cfun(x,y)
    d = dfun(x,y)
    e = efun(x,y)
    f = ffun(x,y)

    h = one/(meshpoints+one)

    stencil(center)    = 2.0_real8*(a + b) - e*h*h
    stencil(left)      =  (half*c*h - a)
    stencil(down)      =  (half*d*h - b)
    stencil(right)     = -(half*c*h + a)
    stencil(up)        = -(half*d*h + b)
    stencil(otherside) = -h*h*f

end subroutine getsten
